@@ -9,21 +9,13 @@
 #include <config.h>
 #endif
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 
 #include "soup-server.h"
-#include "soup-address.h"
-#include "soup-auth-domain.h"
-#include "soup-date.h"
-#include "soup-form.h"
-#include "soup-headers.h"
+#include "soup.h"
 #include "soup-message-private.h"
 #include "soup-marshal.h"
 #include "soup-path-map.h" 
-#include "soup-socket.h"
 
 /**
  * SECTION:soup-server
@@ -134,14 +126,6 @@ enum {
 	LAST_PROP
 };
 
-static GObject *constructor (GType                  type,
-			     guint                  n_construct_properties,
-			     GObjectConstructParam *construct_properties);
-static void set_property (GObject *object, guint prop_id,
-			  const GValue *value, GParamSpec *pspec);
-static void get_property (GObject *object, guint prop_id,
-			  GValue *value, GParamSpec *pspec);
-
 static SoupClientContext *soup_client_context_ref (SoupClientContext *client);
 static void soup_client_context_unref (SoupClientContext *client);
 
@@ -161,24 +145,20 @@ soup_server_init (SoupServer *server)
 }
 
 static void
-finalize (GObject *object)
+soup_server_finalize (GObject *object)
 {
 	SoupServer *server = SOUP_SERVER (object);
 	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (server);
-	GSList *iter;
 
-	if (priv->iface)
-		g_object_unref (priv->iface);
+	g_clear_object (&priv->iface);
 
 	g_free (priv->ssl_cert_file);
 	g_free (priv->ssl_key_file);
-	if (priv->ssl_cert)
-		g_object_unref (priv->ssl_cert);
+	g_clear_object (&priv->ssl_cert);
 
 	g_free (priv->server_header);
 
-	if (priv->listen_sock)
-		g_object_unref (priv->listen_sock);
+	g_clear_object (&priv->listen_sock);
 
 	while (priv->clients) {
 		SoupClientContext *client = priv->clients->data;
@@ -204,22 +184,170 @@ finalize (GObject *object)
 		soup_client_context_unref (client);
 	}
 
-	if (priv->default_handler)
-		free_handler (priv->default_handler);
+	g_clear_pointer (&priv->default_handler, free_handler);
 	soup_path_map_free (priv->handlers);
 
-	for (iter = priv->auth_domains; iter; iter = iter->next)
-		g_object_unref (iter->data);
-	g_slist_free (priv->auth_domains);
+	g_slist_free_full (priv->auth_domains, g_object_unref);
 
-	if (priv->loop)
-		g_main_loop_unref (priv->loop);
-	if (priv->async_context)
-		g_main_context_unref (priv->async_context);
+	g_clear_pointer (&priv->loop, g_main_loop_unref);
+	g_clear_pointer (&priv->async_context, g_main_context_unref);
 
 	G_OBJECT_CLASS (soup_server_parent_class)->finalize (object);
 }
 
+static GObject *
+soup_server_constructor (GType                  type,
+			 guint                  n_construct_properties,
+			 GObjectConstructParam *construct_properties)
+{
+	GObject *server;
+	SoupServerPrivate *priv;
+
+	server = G_OBJECT_CLASS (soup_server_parent_class)->constructor (
+		type, n_construct_properties, construct_properties);
+	if (!server)
+		return NULL;
+	priv = SOUP_SERVER_GET_PRIVATE (server);
+
+	if (!priv->iface) {
+		priv->iface =
+			soup_address_new_any (SOUP_ADDRESS_FAMILY_IPV4,
+					      priv->port);
+	}
+
+	if (priv->ssl_cert_file && priv->ssl_key_file) {
+		GError *error = NULL;
+
+		if (priv->ssl_cert)
+			g_object_unref (priv->ssl_cert);
+		priv->ssl_cert = g_tls_certificate_new_from_files (priv->ssl_cert_file, priv->ssl_key_file, &error);
+		if (!priv->ssl_cert) {
+			g_warning ("Could not read SSL certificate from '%s': %s",
+				   priv->ssl_cert_file, error->message);
+			g_error_free (error);
+			g_object_unref (server);
+			return NULL;
+		}
+	}
+
+	priv->listen_sock =
+		soup_socket_new (SOUP_SOCKET_LOCAL_ADDRESS, priv->iface,
+				 SOUP_SOCKET_SSL_CREDENTIALS, priv->ssl_cert,
+				 SOUP_SOCKET_ASYNC_CONTEXT, priv->async_context,
+				 NULL);
+	if (!soup_socket_listen (priv->listen_sock)) {
+		g_object_unref (server);
+		return NULL;
+	}
+
+	/* Re-resolve the interface address, in particular in case
+	 * the passed-in address had SOUP_ADDRESS_ANY_PORT.
+	 */
+	g_object_unref (priv->iface);
+	priv->iface = soup_socket_get_local_address (priv->listen_sock);
+	g_object_ref (priv->iface);
+	priv->port = soup_address_get_port (priv->iface);
+
+	return server;
+}
+
+static void
+soup_server_set_property (GObject *object, guint prop_id,
+			  const GValue *value, GParamSpec *pspec)
+{
+	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (object);
+	const char *header;
+
+	switch (prop_id) {
+	case PROP_PORT:
+		priv->port = g_value_get_uint (value);
+		break;
+	case PROP_INTERFACE:
+		if (priv->iface)
+			g_object_unref (priv->iface);
+		priv->iface = g_value_get_object (value);
+		if (priv->iface)
+			g_object_ref (priv->iface);
+		break;
+	case PROP_SSL_CERT_FILE:
+		priv->ssl_cert_file =
+			g_strdup (g_value_get_string (value));
+		break;
+	case PROP_SSL_KEY_FILE:
+		priv->ssl_key_file =
+			g_strdup (g_value_get_string (value));
+		break;
+	case PROP_TLS_CERTIFICATE:
+		if (priv->ssl_cert)
+			g_object_unref (priv->ssl_cert);
+		priv->ssl_cert = g_value_dup_object (value);
+		break;
+	case PROP_ASYNC_CONTEXT:
+		priv->async_context = g_value_get_pointer (value);
+		if (priv->async_context)
+			g_main_context_ref (priv->async_context);
+		break;
+	case PROP_RAW_PATHS:
+		priv->raw_paths = g_value_get_boolean (value);
+		break;
+	case PROP_SERVER_HEADER:
+		g_free (priv->server_header);
+		header = g_value_get_string (value);
+		if (!header)
+			priv->server_header = NULL;
+		else if (!*header) {
+			priv->server_header =
+				g_strdup (SOUP_SERVER_SERVER_HEADER_BASE);
+		} else if (g_str_has_suffix (header, " ")) {
+			priv->server_header =
+				g_strdup_printf ("%s%s", header,
+						 SOUP_SERVER_SERVER_HEADER_BASE);
+		} else
+			priv->server_header = g_strdup (header);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+soup_server_get_property (GObject *object, guint prop_id,
+			  GValue *value, GParamSpec *pspec)
+{
+	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (object);
+
+	switch (prop_id) {
+	case PROP_PORT:
+		g_value_set_uint (value, priv->port);
+		break;
+	case PROP_INTERFACE:
+		g_value_set_object (value, priv->iface);
+		break;
+	case PROP_SSL_CERT_FILE:
+		g_value_set_string (value, priv->ssl_cert_file);
+		break;
+	case PROP_SSL_KEY_FILE:
+		g_value_set_string (value, priv->ssl_key_file);
+		break;
+	case PROP_TLS_CERTIFICATE:
+		g_value_set_object (value, priv->ssl_cert);
+		break;
+	case PROP_ASYNC_CONTEXT:
+		g_value_set_pointer (value, priv->async_context ? g_main_context_ref (priv->async_context) : NULL);
+		break;
+	case PROP_RAW_PATHS:
+		g_value_set_boolean (value, priv->raw_paths);
+		break;
+	case PROP_SERVER_HEADER:
+		g_value_set_string (value, priv->server_header);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 static void
 soup_server_class_init (SoupServerClass *server_class)
 {
@@ -228,10 +356,10 @@ soup_server_class_init (SoupServerClass *server_class)
 	g_type_class_add_private (server_class, sizeof (SoupServerPrivate));
 
 	/* virtual method override */
-	object_class->constructor = constructor;
-	object_class->finalize = finalize;
-	object_class->set_property = set_property;
-	object_class->get_property = get_property;
+	object_class->constructor = soup_server_constructor;
+	object_class->finalize = soup_server_finalize;
+	object_class->set_property = soup_server_set_property;
+	object_class->get_property = soup_server_get_property;
 
 	/* signals */
 
@@ -503,159 +631,6 @@ soup_server_class_init (SoupServerClass *server_class)
 				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 }
 
-static GObject *
-constructor (GType                  type,
-	     guint                  n_construct_properties,
-	     GObjectConstructParam *construct_properties)
-{
-	GObject *server;
-	SoupServerPrivate *priv;
-
-	server = G_OBJECT_CLASS (soup_server_parent_class)->constructor (
-		type, n_construct_properties, construct_properties);
-	if (!server)
-		return NULL;
-	priv = SOUP_SERVER_GET_PRIVATE (server);
-
-	if (!priv->iface) {
-		priv->iface =
-			soup_address_new_any (SOUP_ADDRESS_FAMILY_IPV4,
-					      priv->port);
-	}
-
-	if (priv->ssl_cert_file && priv->ssl_key_file) {
-		GError *error = NULL;
-
-		if (priv->ssl_cert)
-			g_object_unref (priv->ssl_cert);
-		priv->ssl_cert = g_tls_certificate_new_from_files (priv->ssl_cert_file, priv->ssl_key_file, &error);
-		if (!priv->ssl_cert) {
-			g_warning ("Could not read SSL certificate from '%s': %s",
-				   priv->ssl_cert_file, error->message);
-			g_error_free (error);
-			g_object_unref (server);
-			return NULL;
-		}
-	}
-
-	priv->listen_sock =
-		soup_socket_new (SOUP_SOCKET_LOCAL_ADDRESS, priv->iface,
-				 SOUP_SOCKET_SSL_CREDENTIALS, priv->ssl_cert,
-				 SOUP_SOCKET_ASYNC_CONTEXT, priv->async_context,
-				 NULL);
-	if (!soup_socket_listen (priv->listen_sock)) {
-		g_object_unref (server);
-		return NULL;
-	}
-
-	/* Re-resolve the interface address, in particular in case
-	 * the passed-in address had SOUP_ADDRESS_ANY_PORT.
-	 */
-	g_object_unref (priv->iface);
-	priv->iface = soup_socket_get_local_address (priv->listen_sock);
-	g_object_ref (priv->iface);
-	priv->port = soup_address_get_port (priv->iface);
-
-	return server;
-}
-
-static void
-set_property (GObject *object, guint prop_id,
-	      const GValue *value, GParamSpec *pspec)
-{
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (object);
-	const char *header;
-
-	switch (prop_id) {
-	case PROP_PORT:
-		priv->port = g_value_get_uint (value);
-		break;
-	case PROP_INTERFACE:
-		if (priv->iface)
-			g_object_unref (priv->iface);
-		priv->iface = g_value_get_object (value);
-		if (priv->iface)
-			g_object_ref (priv->iface);
-		break;
-	case PROP_SSL_CERT_FILE:
-		priv->ssl_cert_file =
-			g_strdup (g_value_get_string (value));
-		break;
-	case PROP_SSL_KEY_FILE:
-		priv->ssl_key_file =
-			g_strdup (g_value_get_string (value));
-		break;
-	case PROP_TLS_CERTIFICATE:
-		if (priv->ssl_cert)
-			g_object_unref (priv->ssl_cert);
-		priv->ssl_cert = g_value_dup_object (value);
-		break;
-	case PROP_ASYNC_CONTEXT:
-		priv->async_context = g_value_get_pointer (value);
-		if (priv->async_context)
-			g_main_context_ref (priv->async_context);
-		break;
-	case PROP_RAW_PATHS:
-		priv->raw_paths = g_value_get_boolean (value);
-		break;
-	case PROP_SERVER_HEADER:
-		g_free (priv->server_header);
-		header = g_value_get_string (value);
-		if (!header)
-			priv->server_header = NULL;
-		else if (!*header) {
-			priv->server_header =
-				g_strdup (SOUP_SERVER_SERVER_HEADER_BASE);
-		} else if (g_str_has_suffix (header, " ")) {
-			priv->server_header =
-				g_strdup_printf ("%s%s", header,
-						 SOUP_SERVER_SERVER_HEADER_BASE);
-		} else
-			priv->server_header = g_strdup (header);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
-static void
-get_property (GObject *object, guint prop_id,
-	      GValue *value, GParamSpec *pspec)
-{
-	SoupServerPrivate *priv = SOUP_SERVER_GET_PRIVATE (object);
-
-	switch (prop_id) {
-	case PROP_PORT:
-		g_value_set_uint (value, priv->port);
-		break;
-	case PROP_INTERFACE:
-		g_value_set_object (value, priv->iface);
-		break;
-	case PROP_SSL_CERT_FILE:
-		g_value_set_string (value, priv->ssl_cert_file);
-		break;
-	case PROP_SSL_KEY_FILE:
-		g_value_set_string (value, priv->ssl_key_file);
-		break;
-	case PROP_TLS_CERTIFICATE:
-		g_value_set_object (value, priv->ssl_cert);
-		break;
-	case PROP_ASYNC_CONTEXT:
-		g_value_set_pointer (value, priv->async_context ? g_main_context_ref (priv->async_context) : NULL);
-		break;
-	case PROP_RAW_PATHS:
-		g_value_set_boolean (value, priv->raw_paths);
-		break;
-	case PROP_SERVER_HEADER:
-		g_value_set_string (value, priv->server_header);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
 /**
  * soup_server_new:
  * @optname1: name of first property to set
@@ -963,8 +938,13 @@ start_request (SoupServer *server, SoupClientContext *client)
 		       msg, client);
 
 	g_object_ref (client->sock);
+
+	if (priv->async_context)
+		g_main_context_push_thread_default (priv->async_context);
 	soup_message_read_request (msg, client->sock,
 				   request_finished, client);
+	if (priv->async_context)
+		g_main_context_pop_thread_default (priv->async_context);
 }
 
 static void
@@ -1146,20 +1126,7 @@ soup_server_get_async_context (SoupServer *server)
  * also be of use in some situations (eg, tracking when multiple
  * requests are made on the same connection).
  **/
-GType
-soup_client_context_get_type (void)
-{
-	static volatile gsize type_volatile = 0;
-
-	if (g_once_init_enter (&type_volatile)) {
-		GType type = g_boxed_type_register_static (
-			g_intern_static_string ("SoupClientContext"),
-			(GBoxedCopyFunc) soup_client_context_ref,
-			(GBoxedFreeFunc) soup_client_context_unref);
-		g_once_init_leave (&type_volatile, type);
-	}
-	return type_volatile;
-}
+G_DEFINE_BOXED_TYPE (SoupClientContext, soup_client_context, soup_client_context_ref, soup_client_context_unref)
 
 /**
  * soup_client_context_get_socket:
